#!/bin/bash

# Some formatting
RED="\033[0;31m"
NC="\033[0m" # No Color

RUNDIR=$(pwd)
PARAM_FILE=${RUNDIR}/param.cfg

# Source parameter with config paths
if [[ ! -f ${PARAM_FILE} ]]
then
   echo -e "No param file "${PARAM_FILE} "found\n${RED}Abort ...${NC}"
   exit 1
else 
  echo "Sourcing param files"
  source ${PARAM_FILE}

  #Check folder on HPC in which the remote implementation will run
  echo -n "Checking remote ..."
  ssh ${HPC_USER}@${HPC_MACHINE} '[ -d '${REMOTE_DIR}' ] && echo -e "... ${RED}ok${NC}" || echo  "Remote dir " '${REMOTE_DIR}' "does not exists" && exit 1' 
  
  REMOTE_DIR=${REMOTE_DIR}/Remote
  
  # Check if given path exists
  if [[ ! -d ${DWARF_HOME} ]]
  then
    echo -e "The specified folder does not exist: "${DWARF_HOME}"\n${RED}Abort${NC}"
    exit 1 
  else
    DWARF_SRC=${DWARF_HOME}/src
    DWARF_NAMELIST=${DWARF_HOME}/escape-acraneb2-input-output/nam_radia_dwarf.nam
  fi
fi

# Folder which contains files for remote compiling
REMOTE=${RUNDIR//Local}Remote
DWARF_REMOTE_HOME=${REMOTE}/${DWARF_HOME##*/}
DWARF_REMOTE_SRC=${DWARF_REMOTE_HOME}/${DWARF_SRC##*/}
DWARF_REMOTE_NAMELIST=${DWARF_REMOTE_HOME}/${DWARF_NAMELIST##*${DWARF_HOME}/}

# Remove eventual leftovers of old runs
rm -rf   ${DWARF_REMOTE_HOME} ${DWARF_HOME##*/} # AutoRPE 

#download last AutoRPE version
#git clone http://earth.bsc.es/gitlab/otinto/AutoRPE.git --branch dwarf-lonlev-0.91_0

AUTORP_HOME=${RUNDIR}/AutoRPE

if [[ ! -d ${AUTORP_HOME} ]]
then
  echo -e "AutoRPE folder not found in : "${AUTORP_HOME}"\n${RED}Abort${NC}"
  exit 1
else 
  # Paths hardcoded in Implement_sources
  AUTO_RP_IN=${AUTORP_HOME}/ModelSources/Input/
  AUTO_RP_OUT=${AUTORP_HOME}/ModelSources/Output/

  # Paths harcoded in NamelistPrecisionsGenerator script
  OLD_NAMELIST=${AUTORP_HOME}/ParameterNamelists/Original/
  NEW_NAMELIST=${AUTORP_HOME}/ParameterNamelists/Modified/
fi

# ---- Start running the emulator ---- 

# Precompile and copy F90 files
for file in ${DWARF_SRC}/*.F90
do
 dest_file=${file//${DWARF_SRC}}
 cpp -P -fopenmp  ${file} > ${AUTO_RP_IN}/${dest_file//F90}f90
done

# Copy .f90 files
cp ${DWARF_SRC}/*.f90 ${AUTO_RP_IN}

# Remove legacy files
rm ${AUTO_RP_IN}/*_legacy*

# Ignore opmpi files and timings, which contains no calculation at all
echo -e  \
 'dmi_omp.F90' \
 '\nini_modules.f90' \
 '\ninput_acraneb2.f90' \
 '\noutput_basic.f90' \
 '\ndmi_omp.f90' \
 '\ndmi_timer.f90' > ${AUTORP_HOME}/AdditionalFiles/list_of_files_to_keep_unmodified.txt


# Check python is loaded
python3 -V  >/dev/null 2>&1  ||  { echo "load python3 and try again ..."; exit 1 ; }

cd ${AUTORP_HOME}

# Run the implementation
python3 ImplementRPEtoSource.py 2> implementation.log

# Add call to read variables precisions to mail file 
sed -i "s/ call domp_init(nthreads)/ call domp_init(nthreads)\n CALL read_variable_precisions/g" ${AUTO_RP_OUT}/main.f90 

#Copy processed files
cp -r ${DWARF_HOME} ${REMOTE} || exit 1
cp ${AUTO_RP_OUT}/* ${DWARF_REMOTE_SRC} || exit 1

# Copy namelist original namelist
cp ${DWARF_NAMELIST} ${OLD_NAMELIST} || exit 1

# Generate namelist_precisions
python3 NamelistPrecisionsGenerator.py

# Modify original namelist
python3 AdaptNamelists.py

# Copy the namelists in the remote folder
cp ${NEW_NAMELIST}/* ${DWARF_REMOTE_NAMELIST} || exit 1
cp namelist_precisions  ${DWARF_REMOTE_NAMELIST} || exit 1

#coping files to remote
echo "Copying files to "${HPC_MACHINE}" machine ..."
scp -r ${REMOTE} ${HPC_USER}@${HPC_MACHINE}:${REMOTE_DIR}  2> /dev/null
#
echo "All files has been copied to "${HPC_USER}@${HPC_MACHINE}:${REMOTE_DIR}
echo -e "${RED}Connecting to '${HPC_MACHINE}' ... ${NC}"
ssh -t ${HPC_USER}@${HPC_MACHINE} 'cd '${REMOTE_DIR}'/Remote && ./Remote.cmd ; bash'